from abc import ABC, abstractmethod
from openai import OpenAI
from typing import Generator

class Ai:

    class AiWrapper(ABC):
        
        @abstractmethod
        def call(self, input: str, **kwargs) -> Generator[str, None, None]:
            """
            Any implmentation of this method should take an input string and return a string,
            optionally taking keyword arguments that can be used to pass additional information.
            """
            pass


    class OpenAIWrapper(AiWrapper):
        # overriding abstract method
        def __init__(self, api_key: str) -> None:
            self.api_key = api_key
            self.client = OpenAI()
            self.client.api_key = api_key
            self.model = "gpt-3.5-turbo"

        def call(self, input:str) -> Generator[str, None, None]: # type: ignore
            client = OpenAI()
            client.api_key = self.api_key
            stream = client.chat.completions.create(
                model=self.model,
                messages=[{"role": "user", "content": input}],
                stream=True,
            )
            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ai_alchemy', 'ai_alchemy.core']

package_data = \
{'': ['*']}

install_requires = \
['openai>=1.30.1,<2.0.0',
 'pandas>=2.2.2,<3.0.0',
 'pydantic>=2.7.1,<3.0.0',
 'requests>=2.31.0,<3.0.0']

setup_kwargs = {
    'name': 'ai-alchemy',
    'version': '0.1.5',
    'description': 'Lightweight package for arbitrary data transformation and validation using AI models and first class python libraries like Pandas and Pydantic.',
    'long_description': '# AI Alchemy\n\nAI Alchemy is a Python library that provides a convenient way to interact with AI models, such as OpenAI\'s GPT-3.5 Turbo, and perform transformations on data.\n\n## Getting Started\n\nThese instructions will get you a copy of the project up and running on your local machine for development and testing purposes.\n\n### Prerequisites\n\nYou need to have Python installed on your machine. You can download Python [here](https://www.python.org/downloads/).\n\n### Installation\n\nYou can install AI Alchemy via pip:\n\n```bash\npip install ai_alchemy\n```\n\n### Usage\nHere\'s a basic example of how to use AI Alchemy:\n\n```python\n# Import necessary libraries\nimport os\nimport ai_alchemy\nfrom ai_alchemy.ai import OpenAIWrapper\nfrom pydantic import BaseModel\n\n# Instantiate a wrapper for an AI model\nopenai = OpenAIWrapper(api_key=os.environ["OPENAI_API_KEY"], model="gpt-3.5-turbo")\n\n# Define a Pydantic model\nclass User(BaseModel):\n    name: str\n    age: int\n\n# Input data\ndata = "John Smith is 25 years old, five foot ten inches tall, and weighs 150 pounds."\n\n# Use AI Alchemy to transform the data into a Pydantic model\nmodel = ai_alchemy.cast.str_to_pydantic_model(data, openai, User)\n\n# Now `model` is a `User` instance with `name` and `age` populated from `data`\n```',
    'author': 'Josh Mogil',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from typing import Type, Any
from pandas import DataFrame
from pydantic import BaseModel

from ai_alchemy.core.ai import AiWrapper


class Cast:
    @staticmethod
    def dict_to_pydantic_model(input: dict, ai: AiWrapper, output: BaseModel):
        base_prompt = [
            "Given the following Pydantic schema and input dictionary, your task is to transform the dictionary into a JSON object. The JSON object should match the Pydantic schema exactly, and should not include the schema itself.",
            "Pydantic schema:",
            f"{output.model_json_schema()}",
            "Input dictionary:",
            f"{input}",
            "Please provide the transformed JSON object."
        ]
            
        joined_prompt="\n".join(base_prompt)
        final=""
        for out in ai.call("\n".join(joined_prompt)):
            final = final + out
        r_val = output.model_validate_json(final)
        return r_val

    @staticmethod
    def str_to_pydantic_model(input: str, ai: AiWrapper, output: BaseModel):
        base_prompt = [
            "Given the following Pydantic schema and input string, your task is to transform the string into a JSON object. The JSON object should match the Pydantic schema exactly, and should not include the schema itself.",
            "Pydantic schema:",
            f"{output.model_json_schema()}",
            "Input string:",
            f"{input}",
            "Please provide the transformed JSON object."
        ]
            
        joined_prompt="\n".join(base_prompt)
        final=""
        for out in ai.call("\n".join(joined_prompt)):
            final = final + out
        r_val = output.model_validate_json(final)
        return r_val

    @staticmethod
    def pydantic_model_to_pydantic_model(input: BaseModel, ai: AiWrapper, output: BaseModel):
        base_prompt = [
            "Given the following Input data and Output Pydantic schema, your task is to transform the Input data a into a JSON object that should match the Pydantic schema exactly, and should not include the schema itself.",
            "Pydantic schema:",
            f"{output.model_json_schema()}",
            "Input data:",
            f"{input.model_dump_json()}",
            "Please provide the transformed JSON object."
        ]
            
        joined_prompt="\n".join(base_prompt)
        final=""
        for out in ai.call("\n".join(joined_prompt)):
            final = final + out
        r_val = output.model_validate_json(final)
        return r_val

def _pydantic_model_to_dataframe(input: BaseModel, ai: AiWrapper, output: DataFrame):
    pass



def _magic(input_data: Any, output_type: Type) -> Any:
    transformation_function = _get_specific_transformation_function(input_data, output_type)


    pass


def _get_specific_transformation_function(input:Type, output:Type):
    pass

    #if isinstance(input, str) and output == str:
    #    return str_to_str
    #elif isinstance(input, str) and issubclass(output, BaseModel):
    #    return str_to_pydantic_model
    #elif isinstance(input, BaseModel) and output == str:
    #    return pydantic_model_to_str
    #elif isinstance(input, BaseModel) and issubclass(output, BaseModel):
    #    return pydantic_model_to_pydantic_model
    #elif isinstance(input, DataFrame) and type(output) == DataFrame:
    #    return dataframe_to_dataframe
    #elif isinstance(input, DataFrame) and issubclass(output, BaseModel):
    #    return dataframe_to_pydantic_model
    #elif isinstance(input, BaseModel) and type(output) == DataFrame:
    #    return pydantic_model_to_dataframe
    #else:
    #    raise ValueError(f"Unsupported transformation {type(input)} -> {type(output)}")



#def str_to_str(input: str, output: str):
#    pass

#def str_to_pydantic_model(input: str, output: BaseModel):
#    pass

#def pydantic_model_to_str(input: BaseModel, output: str):
#    pass



#def dataframe_to_dataframe(input: DataFrame, output: DataFrame):
#    pass

#def dataframe_to_pydantic_model(input: DataFrame, output: BaseModel):
#    pass



#if __name__ == "__main__":
#    _register_transformation_functions()
#    print(TRANSFORMATION_FUNCTIONS)
